/*
Nome: APURAC_RELGESDIRETATUR
Descrio:
Procedimento para preparao de dados que sero necessrios
para os relatrios do contrato de GESTO DIRETA CONSIDERANDO TURNO.

Criao:
18/06/13 - Rodrigo Oliveira 

Manutenes:
__/__/__ _________ - _____________________________________________
__/__/__ _________ - _____________________________________________
__/__/__ _________ - _____________________________________________
__/__/__ _________ - _____________________________________________
*/
create or replace
PROCEDURE APURAC_RELGESDIRETATUR (
      P_CDFILIAL    IN FILIAL.CDFILIAL%TYPE,
      P_DATAINI     IN DATE,
      P_DATAFIN     IN DATE,
      P_NRAPURACGD  IN TRAB_APURAC_GD.NRAPURACGD%TYPE,
      P_CDCLIENTE   IN CLIENTE.CDCLIENTE%TYPE DEFAULT NULL) AS

   /* DECLARAO DO TIPO (RECORD) QUE CONTERA OS DADOS DO CURSOR */
   TYPE GDiretaRecType IS RECORD (
      CdFilial       FILIAL.CDFILIAL%TYPE,
      CdCliente      CLIENTE.CDCLIENTE%TYPE,
      CdServFili     SERVICO.CDSERVFILI%TYPE,
      CDTURNO        EFECLIFATTUR.CDTURNO%TYPE,
      DtFaturam      DATE,
      CdGrServFat    GRUPSERVFAT.CDGRSERVFAT%TYPE,
      CdProduto      PRODUTO.CDPRODUTO%TYPE,
      VrServico      NUMBER,
      VrServFatuDt   NUMBER,
      QtEfetivos     NUMBER);

   /* DECLARAO DO TIPO DO CURSOR */
   TYPE GDiretaCurType IS REF CURSOR RETURN GDiretaRecType;

   /* VARIVEIS QUE CONTERO O CURSOR E O REGISTRO */
   Rec_GDireta    GDiretaRecType;
   Cur_GDireta    GDiretaCurType;
   
   /*CURSOR E REGISTRO QUE RECEBER O ITEM DE DIFERENA DE EFETIVOS NA NOTA FISCAL*/
   REC_MINEFETMES  UTIL_APURAC.MINEFETMES_RECTYPE;
   CUR_MINEFETMES  UTIL_APURAC.MINEFETMES_CURTYPE;

   /* VARIVEIS PARA CONTROLE DE PARAMETROS */
   stIdMiniEfet   VARCHAR2(01);
   stIdQtdePrec   VARCHAR2(01);
   stIdUtEfePla   VARCHAR2(01);
   STIDVLRVARDATA VARCHAR2(01);

   /* VARIVEIS PARA CONTROLE DE APURAO */
   QtdeMiniEfet   EFECLIFAT.QTCOMEFATU%TYPE;
   V_CdServAjus   SERVICO.CDSERVFILI%TYPE;
   V_QtEfetReal   FLOAT;
   V_QtEfetServ   FLOAT;
   V_QtEfetGrup   FLOAT;
   V_TtEfetServ   FLOAT;
   V_VrServico    FLOAT;
   V_VrServReal   FLOAT;
   V_VrServFatu   FLOAT;
   V_VrPercCorr   FLOAT;
   V_Existe       FLOAT;
   V_VRSERVFATUDT NUMBER;
   V_CDSERVPREQUI SERVICO.CDSERVPREQUI%TYPE;
   V_VRSERVFAEQ   SERVCLIE.VRSERVFATU%TYPE;
   V_VRSERVTOTAL  SERVCLIE.VRSERVFATU%TYPE;
   V_TESTE        FLOAT;
   V_CDTURNO      VARCHAR2(1);
   V_CDCLIENTE    VARCHAR2(21);

   /* PROCEDIMENTO QUE ABRE O CURSOS DE ACORDO COM ALGUNS PARAMETROS
    PARAMETROS:
    1. CURSOR A SER ABERTO
    2. INDICADOR DO CONTROLE DE MINIMO DE EFETIVOS (POR DIA OU POR PERIODO)
    3. INDICADOR DA UTILIZACAO DE EFETIVOS DO PLANEJAMENTO
    4. INDICADOR DA UTILIZACAO DE GRUPOS DE SERVICOS
    5. INDICADOR DA VERIFICAO DE EFETIVOS POR TURNO*/
   PROCEDURE OpenGDiretaCursor (C_GDireta  IN OUT GDiretaCurType,
                                V_MinEfet  IN VARCHAR2,
                                V_UtPlanej IN VARCHAR2,
                                V_Grupo    IN VARCHAR2,
                                V_UtTurno  IN VARCHAR2) IS
   BEGIN
      OPEN C_GDireta FOR
         SELECT CDFILIAL,  CDCLIENTE,   CDSERVFILI, CDTURNO,
                DTFATURAM, CDGRSERVFAT, CDPRODUTO, VRSERVICO, NVL(VRSERVFATUDT,0) AS VRSERVFATUDT,
                SUM(QTEFETIVOS)
         FROM SUM_EFETIVOS
         WHERE DTFATURAM BETWEEN P_DATAINI AND P_DATAFIN AND
               ((IDFATUPLAN = 'F') OR (V_UtPlanej = 'S')) AND
               (CDFILIAL   = P_CDFILIAL) AND
               (CDCLIENTE  = P_CDCLIENTE OR P_CDCLIENTE IS NULL)
         GROUP BY CDFILIAL, CDCLIENTE, DTFATURAM, CDSERVFILI, CDTURNO,
                  CDGRSERVFAT, CDPRODUTO, VRSERVICO, VRSERVFATUDT
         HAVING (SUM(QTEFETIVOS) <> 0) AND
                (((CDGRSERVFAT IS NULL) AND (V_Grupo = 'N')) OR
                 ((CDGRSERVFAT IS NOT NULL) AND (V_Grupo = 'S')));
   END OpenGDiretaCursor;

BEGIN

   /* APAGA OS REGISTROS DA TRAB_APURAC_GD DE APURACOES ANTERIORES */
   DELETE FROM TRAB_APURAC_GD
   WHERE CDFILIALAPGD = P_CDFILIAL   AND
         NRAPURACGD   = P_NRAPURACGD;

    /* APAGA OS REGISTROS DA TRAB_APURAC_TU DE APURACOES ANTERIORES */
   DELETE FROM TRAB_APURAC_TU
   WHERE CDFILIALAPTU = P_CDFILIAL   AND
         NRAPURACTU   = P_NRAPURACGD;

   /* BUSCA OS PARAMETROS DA PARAMGERAL E PARAMFILIAL */
   SELECT MAX(SUBSTR(DSPARAMFATFI,01,1)), MAX(SUBSTR(DSPARAMFATFI,02,1)),
          MAX(SUBSTR(DSPARAMFATFI,15,1))
   INTO stIdMiniEfet, stIdQtdePrec, stIdUtEfePla
   FROM PARAMFILIAL
   WHERE CDFILIAL = P_CDFILIAL;

   IF (stIdMiniEfet IS NULL) OR (stIdMiniEfet = ' ') THEN
      stIdMiniEfet := '1';
      -- POR DIA
   END IF;
   IF (stIdQtdePrec IS NULL) OR (stIdQtdePrec = ' ') THEN
      stIdQtdePrec := '1';
      -- POR QUANTIDADE
   END IF;
   IF (stIdUtEfePla IS NULL) OR (stIdUtEfePla = ' ') THEN
      stIdUtEfePla := 'N';
      -- NAO UTILIZA EFETIVOS DO PLANEJAMENTO
   END IF;
   SELECT CDCLIENTE
     INTO V_CDCLIENTE
     FROM CLIENFILIAL  
    WHERE CDFILIAL = P_CDFILIAL
      AND IDCLIENPRIN = 'S'; 
   /*ABRE CURSOR PARA DIFERENCA DE MINIMO DE EFETIVOS POR TURNO */   
   /*ABRE CURSOR QUE IRA INSERIR NAS TABELAS TRAB_APURAC A DIFERENCA DE EFETIVOS POR MES  SEM TURNO!*/
   UTIL_APURAC.OPENEFETMINCUR (CUR_MINEFETMES,
                              P_CDFILIAL,
                              P_DATAINI,
                              P_DATAFIN,
                              stIdUtEfePla,
                              ' ');
   LOOP
      FETCH CUR_MINEFETMES INTO REC_MINEFETMES;
      EXIT WHEN CUR_MINEFETMES%NOTFOUND;
         IF REC_MINEFETMES.QTEFETIVOS > 0 THEN
            IF (REC_MINEFETMES.NMTURNO IS NULL) THEN
               /* VERIFICA SE UTILIZA EQUIVALENCIA DE SERVICO*/ 
              /* SELECT NVL(MAX(CDSERVPREQUI),NULL)
                 INTO V_CDSERVPREQUI
                 FROM SERVICO
                WHERE CDSERVFILI = REC_MINEFETMES.CDSERVFILI AND
                      CDFILIAL   = REC_MINEFETMES.CDFILIAL;

               IF V_CDSERVPREQUI IS NOT NULL THEN
                  SELECT NVL(MAX(VRSERVFATU), 0)
                    INTO V_VRSERVFAEQ
                    FROM SERVCLIE
                   WHERE CDCLIENTE  = V_CDCLIENTE   AND
                         CDFILIAL   = REC_MINEFETMES.CDFILIAL    AND
                         CDSERVFILI = V_CDSERVPREQUI;

                  IF V_VRSERVFAEQ <> 0 THEN
      
                     V_VRSERVTOTAL := REC_MINEFETMES.QTEFETIVOS * REC_MINEFETMES.VRSERVICO;
                     V_QTEFETSERV  := ROUND(V_VRSERVTOTAL /V_VRSERVFAEQ);
                     V_VRSERVICO   := V_VRSERVFAEQ;

                     SELECT COUNT(*)
                       INTO V_Existe
                       FROM TRAB_APURAC_EQ
                      WHERE CDFILIALAPEQ   = REC_MINEFETMES.CDFILIAL   AND
                            NRAPURACEQ     = P_NRAPURACGD              AND
                            CDCLIENTEAPEQ  = V_CDCLIENTE               AND
                            CDSERVFILIAPEQ = REC_MINEFETMES.CDSERVFILI AND
                            CDSERVPREQUI   = V_CDSERVPREQUI            AND
                            IDAJUSEFETAPEQ = 'S'                       AND
                            DTAPURACFATEQ  = REC_MINEFETMES.DTBASE     AND
                            ROUND(VRPRECUNITAPEQ,2) = ROUND(V_VRSERVICO,2);

                     IF (V_Existe = 0) THEN
                        INSERT INTO TRAB_APURAC_EQ
                        (CDFILIALAPEQ, NRAPURACEQ, CDCLIENTEAPEQ,
                         CDSERVFILIAPEQ, CDPRODUTOAPEQ,
                         NREFETREALAPEQ, NREFETIVOSAPEQ,
                         VRPRECREALAPEQ, VRPRECUNITAPEQ,
                         IDAJUSEFETAPEQ, DTAPURACFATEQ, CDSERVPREQUI, IDPRODPRINAPEQ)
                        VALUES (REC_MINEFETMES.CDFILIAL, P_NRAPURACGD, V_CDCLIENTE,
                                REC_MINEFETMES.CDSERVFILI, REC_MINEFETMES.CDPRODUTO,
                                REC_MINEFETMES.QTEFETIVOS, V_QTEFETSERV,
                                REC_MINEFETMES.VRSERVICO, V_VRSERVICO,
                                'S',REC_MINEFETMES.DTBASE, V_CDSERVPREQUI, 'N');
 
                     ELSE
                        UPDATE TRAB_APURAC_EQ
                           SET NREFETIVOSAPEQ = NREFETIVOSAPEQ + V_QTEFETSERV
                         WHERE CDFILIALAPEQ   = REC_MINEFETMES.CDFILIAL   AND
                               NRAPURACEQ     = P_NRAPURACGD              AND
                               CDCLIENTEAPEQ  = V_CDCLIENTE               AND
                               CDSERVFILIAPEQ = REC_MINEFETMES.CDSERVFILI AND
                               CDSERVPREQUI   = V_CDSERVPREQUI         AND
                               IDAJUSEFETAPEQ = 'S'                    AND
                               DTAPURACFATEQ  = REC_MINEFETMES.DTBASE  AND
                               ROUND(VRPRECUNITAPEQ,2) = ROUND(V_VRSERVICO,2);

                     END IF;
                  END IF;
               ELSE
           
                  /*SE NAO UTILIZA, ENTAO INSERE NA TRAB_APURAC_GD*/
                  INSERT INTO TRAB_APURAC_GD 
                     (CDFILIALAPGD  ,  NRAPURACGD   , CDCLIENTEAPGD , CDSERVFILIAPGD, 
                      CDPRODUTOAPGD , NREFETREALAPGD, NREFETIVOSAPGD, VRPRECREALAPGD, 
                      VRPRECUNITAPGD, IDAJUSEFETAPGD, DTAPURACFATGD) 
                  VALUES (REC_MINEFETMES.CDFILIAL , P_NRAPURACGD, V_CDCLIENTE,
                          REC_MINEFETMES.CDSERVFILI, REC_MINEFETMES.CDPRODUTO,
                          REC_MINEFETMES.QTEFETIVOS, REC_MINEFETMES.QTEFETIVOS,
                          REC_MINEFETMES.VRSERVICO, REC_MINEFETMES.VRSERVICO,
                          'N',REC_MINEFETMES.DTBASE);
             --  END IF;           
            END IF;
         END IF;
   END LOOP;
   CLOSE CUR_MINEFETMES;
   /* ABRE O CURSOR */
   OpenGDiretaCursor(Cur_GDireta, stIdMiniEfet, stIdUtEfePla, 'N','N');
   LOOP
      FETCH Cur_GDireta INTO Rec_GDireta;
      EXIT WHEN Cur_GDireta%NOTFOUND;

      /* CALCULA O MINIMO DE EFETIVOS */
      QtdeMiniEfet := UTIL_APURAC.CalcEfetMini(Rec_GDireta.CdFilial,
                                               Rec_GDireta.CdCliente,
                                               Rec_GDireta.CdServFili,
                                               Rec_GDireta.CdGrServFat,
                                               Rec_GDireta.DtFaturam);

      V_VrServico  := Rec_GDireta.VrServico;
      V_VrServReal := Rec_GDireta.VrServico;
      V_QtEfetServ := Rec_GDireta.QtEfetivos;
      V_QtEfetReal := Rec_GDireta.QtEfetivos;

      /* VERIFICACAO DO SERVICO COM VALOR VARIAVEL POR DATA */
      SELECT NVL(MAX(IDVLRVARDATA),'N')
      INTO STIDVLRVARDATA
      FROM SERVICO
      WHERE CDSERVFILI = REC_GDIRETA.CDSERVFILI AND
            CDFILIAL   = REC_GDIRETA.CDFILIAL;

      IF STIDVLRVARDATA = 'S' THEN
         IF (REC_GDIRETA.VRSERVFATUDT IS NOT NULL) AND (REC_GDIRETA.VRSERVFATUDT <> 0) THEN
            V_VRSERVICO  := REC_GDIRETA.VRSERVFATUDT;
         END IF;
      END IF;

      IF Rec_GDireta.CdTurno IS NULL THEN
         /* EFETUA O AJUSTE DAS QUANTIDADES OU VALORES DOS SERVICOS */
         IF (stIdMiniEfet = '1') THEN
         -- POR DIA
            IF (stIdQtdePrec = '1') THEN
            -- POR QUANTIDADE
               IF (Rec_GDireta.QtEfetivos < QtdeMiniEfet) THEN
                  V_QtEfetServ := QtdeMiniEfet;
               END IF;
            ELSE
            -- POR PRECO
               UTIL_APURAC.PrecoCorr(Rec_GDireta.CdFilial,
                                     Rec_GDireta.CdCliente,
                                     Rec_GDireta.CdServFili,
                                     Rec_GDireta.CdGrServFat,
                                     V_QtEfetServ,
                                     QtdeMiniEfet,
                                     'S',
                                     V_VrServico,
                                     V_VrPercCorr);
            END IF;
         ELSE
         -- POR PERIODO

            /* VERIFICA SE JA FOI GRAVADO REGISTRO PARA AJUSTE DE EFETIVOS
               POR QUANTIDADE */
            SELECT COUNT(*)
            INTO V_Existe
            FROM TRAB_APURAC_GD
            WHERE CDFILIALAPGD   = Rec_GDireta.CdFilial   AND
                  NRAPURACGD     = P_NRAPURACGD           AND
                  CDCLIENTEAPGD  = Rec_GDireta.CdCliente  AND
                  CDSERVFILIAPGD = Rec_GDireta.CdServFili AND
                  DTAPURACFATGD  = P_DATAFIN+1            AND
                  IDAJUSEFETAPGD = 'S';

            IF (V_Existe = 0) THEN
               /* CALCULA O TOTAL DE EFETIVOS DO PERIODO */
               SELECT SUM(QTEFETIVOS)
               INTO V_TtEfetServ
               FROM SUM_EFETIVOS
               WHERE (DTFATURAM BETWEEN P_DATAINI AND P_DATAFIN) AND
                     ((IDFATUPLAN = 'F') OR (stIdUtEfePla = 'S')) AND
                     CDFILIAL   = Rec_GDireta.CdFilial AND
                     CDCLIENTE  = Rec_GDireta.CdCliente AND
                     CDSERVFILI = Rec_GDireta.CdServFili;

               IF (stIdQtdePrec = '1') THEN
               -- POR QUANTIDADE
                  IF (V_TtEfetServ < QtdeMiniEfet) THEN
                    
                     SELECT MAX(VRSERVFATHIS)
                     INTO V_VrServFatu
                     FROM HISTSERVFATU
                     WHERE CDFILIAL     = Rec_GDireta.CdFilial   AND
                           CDCLIENTE    = Rec_GDireta.CdCliente  AND
                           CDSERVFILI   = Rec_GDireta.CdServFili AND
                           DTALTEFATHIS = (SELECT MAX(DTALTEFATHIS)
                                           FROM HISTSERVFATU H, (SELECT MAX(DTSRVFATUHIS) AS DTSRVFATUHIS
                                                                 FROM HISTSERVFATU
                                                                 WHERE CDFILIAL = Rec_GDireta.CdFilial   
                                                                 AND CDCLIENTE = Rec_GDireta.CdCliente  
                                                                 AND CDSERVFILI = Rec_GDireta.CdServFili 
                                                                 AND DTSRVFATUHIS <= Rec_GDireta.DtFaturam) X
                                          WHERE H.CDFILIAL     = Rec_GDireta.CdFilial   
                                          AND H.CDCLIENTE    = Rec_GDireta.CdCliente  
                                          AND H.CDSERVFILI   = Rec_GDireta.CdServFili 
                                          AND H.DTSRVFATUHIS = X.DTSRVFATUHIS) AND
                           DTSRVFATUHIS = (SELECT MAX(DTSRVFATUHIS) AS DTSRVFATUHIS
                                           FROM HISTSERVFATU
                                           WHERE CDFILIAL = Rec_GDireta.CdFilial   
                                           AND CDCLIENTE = Rec_GDireta.CdCliente  
                                           AND CDSERVFILI = Rec_GDireta.CdServFili 
                                           AND DTSRVFATUHIS <= Rec_GDireta.DtFaturam);
                     IF (V_VrServFatu IS NULL) THEN
                        SELECT VRSERVFATU
                        INTO V_VrServFatu
                        FROM SERVCLIE
                        WHERE CDFILIAL   = Rec_GDireta.CdFilial AND
                              CDCLIENTE  = Rec_GDireta.CdCliente AND
                              CDSERVFILI = Rec_GDireta.CdServFili;

                        IF (V_VrServFatu IS NULL) THEN
                           V_VrServFatu := 0;
                        END IF;
                     END IF;

                     /* GRAVA O AJUSTE DA QUANTIDADE DE EFETIVOS */
                     INSERT INTO TRAB_APURAC_GD
                     (CDFILIALAPGD, NRAPURACGD, CDCLIENTEAPGD,
                      CDSERVFILIAPGD, CDPRODUTOAPGD,
                      NREFETREALAPGD, NREFETIVOSAPGD,
                      VRPRECREALAPGD, VRPRECUNITAPGD,
                      IDAJUSEFETAPGD, DTAPURACFATGD)
                     VALUES (Rec_GDireta.CdFilial, P_NRAPURACGD, Rec_GDireta.CdCliente,
                             Rec_GDireta.CdServFili, Rec_GDireta.CdProduto,
                             (QtdeMiniEfet-V_TtEfetServ), (QtdeMiniEfet-V_TtEfetServ),
                             V_VrServFatu, V_VrServFatu,
                             'S',P_DATAFIN+1);

                  END IF;
               ELSE
               -- POR PRECO
                  IF (V_TtEfetServ < QtdeMiniEfet) THEN
                     UTIL_APURAC.PrecoCorr(Rec_GDireta.CdFilial,
                                           Rec_GDireta.CdCliente,
                                           Rec_GDireta.CdServFili,
                                           Rec_GDireta.CdGrServFat,
                                           V_TtEfetServ,
                                           QtdeMiniEfet,
                                           'S',
                                           V_VrServico,
                                           V_VrPercCorr);
                  END IF;
               END IF;
            END IF;
         END IF;
      END IF;
      
         IF REC_GDIRETA.CdTurno IS NOT NULL THEN
            /* GRAVA OS SERVICOS NA TRAB_APURAC_TU JA COM AS QUANTIDADES
            E VALORES CORRIGIDOS */
            SELECT COUNT(*)
            INTO V_Existe
            FROM TRAB_APURAC_TU
            WHERE CDFILIALAPTU   = Rec_GDireta.CdFilial   AND
                  NRAPURACTU     = P_NRAPURACGD           AND
                  CDCLIENTEAPTU  = Rec_GDireta.CdCliente  AND
                  CDSERVFILIAPTU = Rec_GDireta.CdServFili AND
                  DTAPURACFATTU  = Rec_GDireta.DtFaturam  AND
                  CDTURNOTU      = REC_GDIRETA.CDTURNO    AND
                  ROUND(VRPRECUNITAPTU,2) = ROUND(V_VrServico,2);

            IF (V_Existe = 0) THEN
               INSERT INTO TRAB_APURAC_TU
               (CDFILIALAPTU, NRAPURACTU, CDCLIENTEAPTU,
                CDSERVFILIAPTU, CDPRODUTOAPTU,
                NREFETREALAPTU, NREFETIVOSAPTU,
                VRPRECREALAPTU, VRPRECUNITAPTU,
                IDAJUSEFETAPTU, DTAPURACFATTU,CDTURNOTU)
               VALUES (Rec_GDireta.CdFilial, P_NRAPURACGD, Rec_GDireta.CdCliente,
                       Rec_GDireta.CdServFili, Rec_GDireta.CdProduto,
                       V_QtEfetReal, V_QtEfetServ,
                       V_VrServReal, V_VrServico,
                       'N',Rec_GDireta.DtFaturam, Rec_GDireta.CdTurno);

            ELSE
               UPDATE TRAB_APURAC_TU
               SET NREFETIVOSAPTU = NREFETIVOSAPTU + V_QtEfetServ
               WHERE CDFILIALAPTU   = Rec_GDireta.CdFilial   AND
                     NRAPURACTU     = P_NRAPURACGD           AND
                     CDCLIENTEAPTU  = Rec_GDireta.CdCliente  AND
                     CDSERVFILIAPTU = Rec_GDireta.CdServFili AND
                     DTAPURACFATTU  = Rec_GDireta.DtFaturam  AND
                     CDTURNOTU      = REC_GDIRETA.CDTURNO    AND
                     ROUND(VRPRECUNITAPTU,2) = ROUND(V_VrServico,2);

            END IF;
         ELSE
            /* GRAVA OS SERVICOS NA TRAB_APURAC_GD JA COM AS QUANTIDADES
               E VALORES CORRIGIDOS */
            SELECT COUNT(*)
            INTO V_Existe
            FROM TRAB_APURAC_GD
            WHERE CDFILIALAPGD   = Rec_GDireta.CdFilial   AND
                  NRAPURACGD     = P_NRAPURACGD           AND
                  CDCLIENTEAPGD  = Rec_GDireta.CdCliente  AND
                  CDSERVFILIAPGD = Rec_GDireta.CdServFili AND
                  CDPRODUTOAPGD  = Rec_GDireta.CdProduto  AND
                  DTAPURACFATGD  = Rec_GDireta.DtFaturam  AND
                  ROUND(VRPRECUNITAPGD,2) = ROUND(V_VrServico,2);

            IF (V_Existe = 0) THEN
               INSERT INTO TRAB_APURAC_GD
               (CDFILIALAPGD, NRAPURACGD, CDCLIENTEAPGD,
                CDSERVFILIAPGD, CDPRODUTOAPGD,
                NREFETREALAPGD, NREFETIVOSAPGD,
                VRPRECREALAPGD, VRPRECUNITAPGD,
                IDAJUSEFETAPGD, DTAPURACFATGD)
               VALUES (Rec_GDireta.CdFilial, P_NRAPURACGD, Rec_GDireta.CdCliente,
                       Rec_GDireta.CdServFili, Rec_GDireta.CdProduto,
                       V_QtEfetReal, V_QtEfetServ,
                       V_VrServReal, V_VrServico,
                       'N',Rec_GDireta.DtFaturam);

            ELSE
               UPDATE TRAB_APURAC_GD
               SET NREFETIVOSAPGD = NREFETIVOSAPGD + V_QtEfetServ
               WHERE CDFILIALAPGD   = Rec_GDireta.CdFilial   AND
                     NRAPURACGD     = P_NRAPURACGD           AND
                     CDCLIENTEAPGD  = Rec_GDireta.CdCliente  AND
                     CDSERVFILIAPGD = Rec_GDireta.CdServFili AND
                     CDPRODUTOAPGD  = Rec_GDireta.CdProduto  AND
                     DTAPURACFATGD  = Rec_GDireta.DtFaturam  AND
                     ROUND(VRPRECUNITAPGD,2) = ROUND(V_VrServico,2);                     

            END IF;
         END IF;
   END LOOP;
   CLOSE Cur_GDireta;

   /* ABRE O CURSOR PARA TODOS OS SERVICOS QUE POSSUEM GRUPOS */
   OpenGDiretaCursor(Cur_GDireta, stIdMiniEfet, stIdUtEfePla, 'S','N');
   LOOP
      FETCH Cur_GDireta INTO Rec_GDireta;
      EXIT WHEN Cur_GDireta%NOTFOUND;

      /* CALCULA O MINIMO DE EFETIVOS */
      QtdeMiniEfet := UTIL_APURAC.CalcEfetMini(Rec_GDireta.CdFilial,
                                               Rec_GDireta.CdCliente,
                                               Rec_GDireta.CdServFili,
                                               Rec_GDireta.CdGrServFat,
                                               Rec_GDireta.DtFaturam);

      V_VrServico  := Rec_GDireta.VrServico;
      V_VrServReal := Rec_GDireta.VrServico;
      V_QtEfetServ := Rec_GDireta.QtEfetivos;
      V_QtEfetReal := Rec_GDireta.QtEfetivos;

      /* VERIFICACAO DO SERVICO COM VALOR VARIAVEL POR DATA */
      SELECT NVL(MAX(IDVLRVARDATA),'N')
      INTO STIDVLRVARDATA
      FROM SERVICO
      WHERE CDSERVFILI = REC_GDIRETA.CDSERVFILI AND
            CDFILIAL   = REC_GDIRETA.CDFILIAL;

      IF STIDVLRVARDATA = 'S' THEN
         IF (REC_GDIRETA.VRSERVFATUDT IS NOT NULL) AND (REC_GDIRETA.VRSERVFATUDT <> 0)THEN
            V_VRSERVICO  := REC_GDIRETA.VRSERVFATUDT;
         END IF;
      END IF;

      IF REC_GDIRETA.CdTurno IS NULL THEN
          SELECT SUM(QTEFETIVOS)
          INTO V_QtEfetGrup
          FROM SUM_EFETIVOS
          WHERE (DTFATURAM = Rec_GDireta.DtFaturam) AND
                ((IDFATUPLAN = 'F') OR (stIdUtEfePla = 'S')) AND
                CDFILIAL    = Rec_GDireta.CdFilial AND
                CDCLIENTE   = Rec_GDireta.CdCliente AND
                CDGRSERVFAT = Rec_GDireta.CdGrServFat;

          SELECT CDSERAJEFEGSF
          INTO V_CdServAjus
          FROM GRUPSERVFAT
          WHERE CDFILIAL    = Rec_GDireta.CdFilial AND
                CDCLIENTE   = Rec_GDireta.CdCliente AND
                CDGRSERVFAT = Rec_GDireta.CdGrServFat;

          /* EFETUA O AJUSTE DAS QUANTIDADES OU VALORES DOS SERVICOS */
          IF (stIdMiniEfet = '1') THEN
          -- POR DIA
             IF (stIdQtdePrec = '1') THEN
             -- POR QUANTIDADE
                IF (V_QtEfetGrup < QtdeMiniEfet) AND (Rec_GDireta.CdServFili = V_CdServAjus) THEN
                   V_QtEfetServ := V_QtEfetServ + (QtdeMiniEfet - V_QtEfetGrup);
                END IF;
             ELSE
             -- POR PRECO
                UTIL_APURAC.PrecoCorr(Rec_GDireta.CdFilial,
                                      Rec_GDireta.CdCliente,
                                      Rec_GDireta.CdServFili,
                                      Rec_GDireta.CdGrServFat,
                                      V_QtEfetGrup,
                                      QtdeMiniEfet,
                                      'G',
                                      V_VrServico,
                                      V_VrPercCorr);
             END IF;
          ELSE
          -- POR PERIODO

             /* VERIFICA SE JA FOI GRAVADO REGISTRO PARA AJUSTE DE EFETIVOS
                POR QUANTIDADE */
             SELECT COUNT(*)
             INTO V_Existe
             FROM TRAB_APURAC_GD
             WHERE CDFILIALAPGD   = Rec_GDireta.CdFilial   AND
                   NRAPURACGD     = P_NRAPURACGD           AND
                   CDCLIENTEAPGD  = Rec_GDireta.CdCliente  AND
                   CDSERVFILIAPGD = Rec_GDireta.CdServFili AND
                   DTAPURACFATGD  = P_DATAFIN+1            AND
                   IDAJUSEFETAPGD = 'S';

             IF (V_Existe = 0) THEN
                /* CALCULA O TOTAL DE EFETIVOS DO PERIODO */
                SELECT SUM(QTEFETIVOS)
                INTO V_QtEfetGrup
                FROM SUM_EFETIVOS
                WHERE (DTFATURAM BETWEEN P_DATAINI AND P_DATAFIN) AND
                      ((IDFATUPLAN = 'F') OR (stIdUtEfePla = 'S')) AND
                      CDFILIAL    = Rec_GDireta.CdFilial AND
                      CDCLIENTE   = Rec_GDireta.CdCliente AND
                      CDGRSERVFAT = Rec_GDireta.CdGrServFat;

                IF (stIdQtdePrec = '1') THEN
                -- POR QUANTIDADE
                   IF (V_QtEfetGrup < QtdeMiniEfet) AND (Rec_GDireta.CdServFili = V_CdServAjus) THEN
                      SELECT MAX(VRSERVFATHIS)
                      INTO V_VrServFatu
                        FROM HISTSERVFATU
                       WHERE CDFILIAL     = Rec_GDireta.CdFilial   AND
                             CDCLIENTE    = Rec_GDireta.CdCliente  AND
                             CDSERVFILI   = Rec_GDireta.CdServFili AND
                             DTALTEFATHIS = (SELECT MAX(DTALTEFATHIS)
                                             FROM HISTSERVFATU H, (SELECT MAX(DTSRVFATUHIS) AS DTSRVFATUHIS
                                                                   FROM HISTSERVFATU
                                                                   WHERE CDFILIAL = Rec_GDireta.CdFilial   
                                                                   AND CDCLIENTE = Rec_GDireta.CdCliente  
                                                                   AND CDSERVFILI = Rec_GDireta.CdServFili 
                                                                   AND DTSRVFATUHIS <= Rec_GDireta.DtFaturam) X
                                            WHERE H.CDFILIAL     = Rec_GDireta.CdFilial   
                                            AND H.CDCLIENTE    = Rec_GDireta.CdCliente  
                                            AND H.CDSERVFILI   = Rec_GDireta.CdServFili 
                                            AND H.DTSRVFATUHIS = X.DTSRVFATUHIS) AND
                             DTSRVFATUHIS = (SELECT MAX(DTSRVFATUHIS) AS DTSRVFATUHIS
                                             FROM HISTSERVFATU
                                             WHERE CDFILIAL = Rec_GDireta.CdFilial   
                                             AND CDCLIENTE = Rec_GDireta.CdCliente  
                                             AND CDSERVFILI = Rec_GDireta.CdServFili 
                                             AND DTSRVFATUHIS <= Rec_GDireta.DtFaturam);

                      IF (V_VrServFatu IS NULL) THEN
                         SELECT VRSERVFATU
                         INTO V_VrServFatu
                         FROM SERVCLIE
                         WHERE CDFILIAL   = Rec_GDireta.CdFilial AND
                               CDCLIENTE  = Rec_GDireta.CdCliente AND
                               CDSERVFILI = Rec_GDireta.CdServFili;

                         IF (V_VrServFatu IS NULL) THEN
                            V_VrServFatu := 0;
                         END IF;
                      END IF;

                      /* GRAVA O AJUSTE DA QUANTIDADE DE EFETIVOS */
                      INSERT INTO TRAB_APURAC_GD
                      (CDFILIALAPGD, NRAPURACGD, CDCLIENTEAPGD,
                       CDSERVFILIAPGD, CDPRODUTOAPGD,
                       NREFETREALAPGD, NREFETIVOSAPGD,
                       VRPRECREALAPGD, VRPRECUNITAPGD,
                       IDAJUSEFETAPGD, DTAPURACFATGD)
                      VALUES (Rec_GDireta.CdFilial, P_NRAPURACGD, Rec_GDireta.CdCliente,
                              Rec_GDireta.CdServFili, Rec_GDireta.CdProduto,
                              (QtdeMiniEfet-V_QtEfetGrup), (QtdeMiniEfet-V_QtEfetGrup),
                              V_VrServFatu, V_VrServFatu,
                              'S',P_DATAFIN+1);

                   END IF;
                ELSE
                -- POR PRECO
                   IF (V_QtEfetGrup < QtdeMiniEfet) THEN
                      UTIL_APURAC.PrecoCorr(Rec_GDireta.CdFilial,
                                            Rec_GDireta.CdCliente,
                                            Rec_GDireta.CdServFili,
                                            Rec_GDireta.CdGrServFat,
                                            V_QtEfetGrup,
                                            QtdeMiniEfet,
                                            'G',
                                            V_VrServico,
                                            V_VrPercCorr);
                   END IF;
                END IF;
             END IF;
          END IF;
      END IF;
         IF REC_GDIRETA.CdTurno IS NOT NULL THEN
            /* GRAVA OS SERVICOS NA TRAB_APURAC_TU JA COM AS QUANTIDADES
            E VALORES CORRIGIDOS */
            SELECT COUNT(*)
            INTO V_Existe
            FROM TRAB_APURAC_TU
            WHERE CDFILIALAPTU   = Rec_GDireta.CdFilial   AND
                  NRAPURACTU     = P_NRAPURACGD           AND
                  CDCLIENTEAPTU  = Rec_GDireta.CdCliente  AND
                  CDSERVFILIAPTU = REC_GDIRETA.CDSERVFILI AND
                  DTAPURACFATTU  = Rec_GDireta.DtFaturam  AND
                  CDTURNOTU      = REC_GDIRETA.CDTURNO    AND
                  ROUND(VRPRECUNITAPTU,2) = ROUND(V_VrServico,2);

            IF (V_Existe = 0) THEN
               INSERT INTO TRAB_APURAC_TU
               (CDFILIALAPTU, NRAPURACTU, CDCLIENTEAPTU,
                CDSERVFILIAPTU, CDPRODUTOAPTU,
                NREFETREALAPTU, NREFETIVOSAPTU,
                VRPRECREALAPTU, VRPRECUNITAPTU,
                IDAJUSEFETAPTU, DTAPURACFATTU,CDTURNOTU)
               VALUES (Rec_GDireta.CdFilial, P_NRAPURACGD, Rec_GDireta.CdCliente,
                       REC_GDIRETA.CDSERVFILI, Rec_GDireta.CdProduto,
                       V_QtEfetReal, V_QtEfetServ,
                       V_VrServReal, V_VrServico,
                       'N',Rec_GDireta.DtFaturam, Rec_GDireta.CdTurno);

            ELSE
               UPDATE TRAB_APURAC_TU
               SET NREFETIVOSAPTU = NREFETIVOSAPTU + V_QtEfetServ
               WHERE CDFILIALAPTU   = Rec_GDireta.CdFilial   AND
                     NRAPURACTU     = P_NRAPURACGD           AND
                     CDCLIENTEAPTU  = Rec_GDireta.CdCliente  AND
                     CDSERVFILIAPTU = REC_GDIRETA.CDSERVFILI AND
                     DTAPURACFATTU  = Rec_GDireta.DtFaturam  AND
                     CDTURNOTU      = REC_GDIRETA.CDTURNO    AND
                     ROUND(VRPRECUNITAPTU,2) = ROUND(V_VrServico,2);

            END IF;
         ELSE

            /* GRAVA OS SERVICOS NA TRAB_APURAC_GD JA COM AS QUANTIDADES
               E VALORES CORRIGIDOS */
            SELECT COUNT(*)
            INTO V_Existe
            FROM TRAB_APURAC_GD
            WHERE CDFILIALAPGD   = Rec_GDireta.CdFilial   AND
                  NRAPURACGD     = P_NRAPURACGD           AND
                  CDCLIENTEAPGD  = Rec_GDireta.CdCliente  AND
                  CDSERVFILIAPGD = REC_GDIRETA.CDSERVFILI AND
                  CDPRODUTOAPGD  = Rec_GDireta.CdProduto  AND
                  DTAPURACFATGD  = Rec_GDireta.DtFaturam  AND
                  ROUND(VRPRECUNITAPGD,2) = ROUND(V_VrServico,2);

            IF (V_Existe = 0) THEN
               INSERT INTO TRAB_APURAC_GD
               (CDFILIALAPGD, NRAPURACGD, CDCLIENTEAPGD,
                CDSERVFILIAPGD, CDPRODUTOAPGD,
                NREFETREALAPGD, NREFETIVOSAPGD,
                VRPRECREALAPGD, VRPRECUNITAPGD,
                IDAJUSEFETAPGD, DTAPURACFATGD)
               VALUES (Rec_GDireta.CdFilial, P_NRAPURACGD, Rec_GDireta.CdCliente,
                       REC_GDIRETA.CDSERVFILI, Rec_GDireta.CdProduto,
                       V_QtEfetReal, V_QtEfetServ,
                       V_VrServReal, V_VrServico,
                       'N',Rec_GDireta.DtFaturam);

            ELSE
               UPDATE TRAB_APURAC_GD
               SET NREFETIVOSAPGD = NREFETIVOSAPGD + V_QtEfetServ
               WHERE CDFILIALAPGD   = Rec_GDireta.CdFilial   AND
                     NRAPURACGD     = P_NRAPURACGD           AND
                     CDCLIENTEAPGD  = Rec_GDireta.CdCliente  AND
                     CDSERVFILIAPGD = REC_GDIRETA.CDSERVFILI AND
                     CDPRODUTOAPGD  = Rec_GDireta.CdProduto  AND
                     DTAPURACFATGD  = Rec_GDireta.DtFaturam  AND
                     ROUND(VRPRECUNITAPGD,2) = ROUND(V_VrServico,2);                     

            END IF;
         END IF;
   END LOOP; 
   Close Cur_Gdireta;
END;
/
BEGIN
 ATUVEROBJ('APURAC_RELGESDIRETATUR','5.65.000');
END;
/
