/* Sistemas Integrados TEKNISA
   Compras e Suprimentos

Nome: 8982_SUPSTPO.sql

Descrio:
Executa a primeira e segunda fase da atualizao do custo de lista de preos.
Fase 1 --> Excluso de custos no condizentes com os cadastros atuais
Fase 2 --> Atualizao de custos cuja cotao no mais existe
Criao:
31/07/2007 Marcelo Alexsandro Souza Silva
*/

CREATE OR REPLACE
PROCEDURE ATU_CUSTO_ITEMCOL_BKG
  (P_CDFILIAL     IN ITMPREVLIST.CDFILIAL%TYPE,
   P_NRSEQPRELIST IN ITMPREVLIST.NRSEQPRELIST%TYPE) IS

   V_DTINIVALLIST PREVLISTA.DTINIVALLIST%TYPE;
   V_DTFINVALLIST PREVLISTA.DTFINVALLIST%TYPE;
   V_CDOPERADOR   PREVLISTA.CDOPERADOR%TYPE;

   V_CONT_COMMIT NUMBER := 0;
   V_QTDE_COMMIT NUMBER := 500;

   V_LISTA_FILIAL VARCHAR2(20); /* NUMERO DA PRXLISTA SOBRE FILIAL */

BEGIN
   BEGIN
      SELECT
         DTINIVALLIST,
         DTFINVALLIST,
         CDOPERADOR
      INTO
         V_DTINIVALLIST,
         V_DTFINVALLIST,
         V_CDOPERADOR
      FROM
         PREVLISTA
      WHERE
         CDFILIAL     = P_CDFILIAL     AND
         NRSEQPRELIST = P_NRSEQPRELIST;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN BEGIN
         V_DTINIVALLIST := NULL;
         V_DTFINVALLIST := NULL;
         V_CDOPERADOR   := NULL;
      END;
   END;

   V_CONT_COMMIT := 0;

   NOVO_CODIGO('LISTA', V_LISTA_FILIAL, 1);
   V_LISTA_FILIAL := SUBSTR(V_LISTA_FILIAL, 11, 10);
   FOR R1 IN (WITH FILIAIS AS
     (SELECT
         DISTINCT C.CDFILIAL
      FROM
         ITEMCOLE B,
         FILIAL C
      WHERE
         P_CDFILIAL = NVL(C.CDFILCENTCMP, C.CDFILIAL) AND
         C.CDREGIAO = B.CDREGIAO      AND
         B.CDFILIAL = P_CDFILIAL      AND
         B.NRSEQPRELIST = P_NRSEQPRELIST)
      SELECT
         V_LISTA_FILIAL,
         F.CDFILIAL
      FROM
         FILIAIS F
      WHERE
         EXISTS (SELECT
                    1
                 FROM
                    CARDADIA C
                 WHERE
                    C.CDFILIAL = F.CDFILIAL)
      ) LOOP
      INSERT INTO LISTA
         (NRLISTA, CDLISTA)
      VALUES
         (V_LISTA_FILIAL, R1.CDFILIAL);
   END LOOP;

   FOR C_VERIF_CUSTO IN
      (SELECT /*+ ORDERED */
          CUSTOSUP.ROWID,
          CUSTOSUP.CDFILIAL,
          CUSTOSUP.CDPRODUTO,
          CUSTOSUP.DTCUSTOSUP
       FROM
          LISTA,
          CUSTOSUP,
          COLETAPR
       WHERE
          COLETAPR.CDFILIAL       = P_CDFILIAL            AND
          COLETAPR.CDOPERADOR     = V_CDOPERADOR          AND
          CUSTOSUP.CDFILCOTLIST   = COLETAPR.CDFILIAL     AND
          CUSTOSUP.NRCOLEPRLIST   = COLETAPR.NRCOLEPRECO  AND
          V_DTINIVALLIST         <= CUSTOSUP.DTCUSTOSUP   AND
          V_DTFINVALLIST         >= CUSTOSUP.DTFINCUSTSUP AND
          LISTA.NRLISTA           = V_LISTA_FILIAL        AND
          CUSTOSUP.CDFILIAL       = LISTA.CDLISTA         AND
          CUSTOSUP.VRPRCUSTOAF IS NULL                    AND
          (EXISTS(SELECT
                     '1'
                  FROM
                     FORNECEDOR
                  WHERE
                     FORNECEDOR.CDFORNECED = CUSTOSUP.CDFORNECLIST AND
                     ((NVL(FORNECEDOR.IDSITUCADA, 'A') <> 'A') OR (NVL(FORNECEDOR.IDTIPOFORN, 'N') <> 'N'))
                  ) OR
           EXISTS(SELECT
                     '1'
                  FROM
                     REFORFIL
                  WHERE
                     (REFORFIL.CDFILIAL   = CUSTOSUP.CDFILIAL)     AND
                     (REFORFIL.CDFORNECED = CUSTOSUP.CDFORNECLIST) AND
                     (REFORFIL.CDPERIENTR IS NULL)
                  ) OR
           EXISTS(SELECT
                     '1'
                  FROM
                     REFOREG
                  WHERE
                    REFOREG.CDREGIAO   = CUSTOSUP.CDREGIAOLIST AND
                    REFOREG.CDFORNECED = CUSTOSUP.CDFORNECLIST
                  ) OR
           EXISTS(SELECT
                     '1'
                  FROM
                     QFOPRODFORN,
                     PARASUPRI
                  WHERE
                     (PARASUPRI.CDFILIAL     = P_CDFILIAL)            AND
                     (QFOPRODFORN.CDPRODUTO  = CUSTOSUP.CDPRODLIST)   AND
                     (QFOPRODFORN.CDFORNECED = CUSTOSUP.CDFORNECLIST) AND
                     ((QFOPRODFORN.IDFORNAPROV = 'N' AND QFOPRODFORN.IDFORNAUTOR = 'N') OR ((QFOPRODFORN.IDFORNAPROV = 'S') AND ((PARASUPRI.IDULTIDTVALQ = 'M' AND COLETAPR.DTCOLEPREC > LAST_DAY(QFOPRODFORN.DTVALPRFORN)) OR (PARASUPRI.IDULTIDTVALQ = 'D' AND COLETAPR.DTCOLEPREC > QFOPRODFORN.DTVALPRFORN))))) OR
           EXISTS(SELECT
                     '1'
                  FROM
                     QFOMARPROD,
                     PARASUPRI
                  WHERE
                     (PARASUPRI.CDFILIAL   = P_CDFILIAL)           AND
                     (QFOMARPROD.CDPRODUTO = CUSTOSUP.CDPRODLIST)  AND
                     (QFOMARPROD.CDMARCA   = CUSTOSUP.CDMARCALIST) AND
                     ((QFOMARPROD.IDAPRMARPRO = 'N' AND QFOMARPROD.IDAUTMARPRO = 'N') OR ((QFOMARPROD.IDAPRMARPRO = 'S') AND ((PARASUPRI.IDULTIDTVALQ = 'M' AND COLETAPR.DTCOLEPREC > LAST_DAY(QFOMARPROD.DTVALAPROVA)) OR (PARASUPRI.IDULTIDTVALQ = 'D' AND COLETAPR.DTCOLEPREC > QFOMARPROD.DTVALAPROVA))))) OR
           NOT EXISTS(SELECT
                        '1'
                     FROM
                        QFOMARPROD
                     WHERE
                        (QFOMARPROD.CDPRODUTO = CUSTOSUP.CDPRODLIST) AND
                        (QFOMARPROD.CDMARCA   = CUSTOSUP.CDMARCALIST)

                     UNION ALL

                     SELECT
                        '1'
                     FROM
                        PARASUPRI
                     WHERE
                        (PARASUPRI.CDFILIAL    = P_CDFILIAL) AND
                        (PARASUPRI.IDMARSEMQUA = 'N' OR PARASUPRI.IDMARSEMQUA IS NULL)
                      ) OR
           EXISTS(SELECT
                     '1'
                  FROM
                     QUALIFICACAO,
                     FORNECEDOR,
                     PARASUPRI
                  WHERE
                     (FORNECEDOR.CDFORNECED     = CUSTOSUP.CDFORNECLIST) AND
                     (QUALIFICACAO.CDQUALIF     = FORNECEDOR.CDQUALIF)   AND
                     (PARASUPRI.CDFILIAL        = P_CDFILIAL)            AND
                     (FORNECEDOR.IDFORNAUTO     = 'N')                   AND
                     ((QUALIFICACAO.IDPERCOTCMP = '2') OR ((PARASUPRI.IDULTIDTVALQ = 'D' AND COLETAPR.DTCOLEPREC > FORNECEDOR.DTVALQUALIF) OR (PARASUPRI.IDULTIDTVALQ = 'M' AND COLETAPR.DTCOLEPREC > LAST_DAY(FORNECEDOR.DTVALQUALIF))))) OR
           ((NOT EXISTS (SELECT
                           '1'
                        FROM
                           PARTFILI
                        WHERE
                           (PARTFILI.CDFILIAL  = CUSTOSUP.CDFILIAL)      AND
                           (PARTFILI.CDPRODUTO = CUSTOSUP.CDPRODLIST)    AND
                           (PARTFILI.CDTIPPART IN ('1', '2'))            AND
                           (PARTFILI.CDMARCA   = CUSTOSUP.CDMARCALIST)))

            AND (NOT EXISTS (SELECT
                                '1'
                             FROM
                                PARTFILI
                             WHERE
                                (PARTFILI.CDFILIAL   = CUSTOSUP.CDFILIAL)   AND
                                (PARTFILI.CDPRODUTO  = CUSTOSUP.CDPRODLIST) AND
                                (PARTFILI.CDTIPPART  IN ('4', '5'))         AND
                                (PARTFILI.CDFORNECED = CUSTOSUP.CDFORNECLIST)))

            AND (NOT EXISTS (SELECT
                                '1'
                             FROM
                                PARTFILI
                             WHERE
                                (PARTFILI.CDFILIAL   = CUSTOSUP.CDFILIAL)      AND
                                (PARTFILI.CDPRODUTO  = CUSTOSUP.CDPRODLIST)    AND
                                (PARTFILI.CDTIPPART  IN ('7', '8'))            AND
                                (PARTFILI.CDMARCA    = CUSTOSUP.CDMARCALIST)   AND
                                (PARTFILI.CDFORNECED = CUSTOSUP.CDFORNECLIST)))

            AND (NOT EXISTS (SELECT
                                '1'
                             FROM
                                PARTFILI
                             WHERE
                                (PARTFILI.CDFILIAL  = CUSTOSUP.CDFILIAL)   AND
                                (PARTFILI.CDPRODUTO = CUSTOSUP.CDPRODLIST) AND
                                (PARTFILI.CDTIPPART  IN ('3','6','9'))))

            AND (EXISTS (SELECT
                            '1'
                         FROM
                            PARTFILI
                         WHERE
                            (PARTFILI.CDFILIAL  = CUSTOSUP.CDFILIAL) AND
                            (PARTFILI.CDPRODUTO = CUSTOSUP.CDPRODLIST)))
           ) OR
           EXISTS (SELECT
                      '1'
                   FROM
                      PARTFILI
                   WHERE
                      (CDFILIAL  = CUSTOSUP.CDFILIAL)     AND
                      (CDPRODUTO = CUSTOSUP.CDPRODLIST)   AND
                      (CDTIPPART = '3')                   AND
                      (CDMARCA   = CUSTOSUP.CDMARCALIST)) OR
           EXISTS (SELECT
                      '1'
                   FROM
                      PARTFILI
                   WHERE
                      (CDFILIAL   = CUSTOSUP.CDFILIAL)  AND
                      (CDPRODUTO  = CUSTOSUP.CDPRODLIST)AND
                      (CDTIPPART  = '6')                AND
                      (CDFORNECED = CUSTOSUP.CDFORNECLIST)) OR
           EXISTS (SELECT
                      '1'
                   FROM
                      PARTFILI
                   WHERE
                      (CDFILIAL   = CUSTOSUP.CDFILIAL)      AND
                      (CDPRODUTO  = CUSTOSUP.CDPRODLIST)    AND
                      (CDTIPPART  = '9')                    AND
                      (CDMARCA    = CUSTOSUP.CDMARCALIST)   AND
                      (CDFORNECED = CUSTOSUP.CDFORNECLIST))
           )
        )
   LOOP
      DELETE FROM CUSTOSUP
      WHERE
        C_VERIF_CUSTO.CDFILIAL   = CUSTOSUP.CDFILIALORIG AND
        C_VERIF_CUSTO.CDPRODUTO  = CUSTOSUP.CDPRODORIG   AND
        C_VERIF_CUSTO.DTCUSTOSUP = CUSTOSUP.DTCUSTSUPORI;

      DELETE FROM CUSTOSUP
      WHERE C_VERIF_CUSTO.ROWID = CUSTOSUP.ROWID;
      COMMIT;
   END LOOP;

   /* Cursor: C_DELCUSTACOR

      Tratamento dos custos de lista de preos da qual a cotao de acordo
      no mais existe. Os campos da cotao de acordo sero limpos e o valor
      do custo ser substitudo pelo valor da lista(atravs do campo VRHISPRLISTP),
      se no houver os campos da lista e nem custo de AF, o registro ser excludo.
   */

   FOR R1 IN (SELECT CDLISTA FROM LISTA WHERE NRLISTA = V_LISTA_FILIAL) LOOP
     FOR C_CUSTOSUP IN
       (WITH CUSTOSUP_X AS /* 1 */
       (SELECT
         CUSTOSUP.ROWID,
         CUSTOSUP.CDFILIAL,
         CUSTOSUP.CDPRODUTO,
         CUSTOSUP.DTCUSTOSUP,
         CUSTOSUP.NRCOLEPRLIST,
         CUSTOSUP.VRPRCUSTOAF,
         CUSTOSUP.CDFILCOTACOR,
         CUSTOSUP.NRCOLEPRACOR,
         CUSTOSUP.CDMARCAACOR,
         CUSTOSUP.CDREGIAOACOR,
         CUSTOSUP.CDFORNECACOR,
         CUSTOSUP.CDPRODACOR,
         CUSTOSUP.CDFILCOTLIST,
         CUSTOSUP.CDMARCALIST,
         CUSTOSUP.CDREGIAOLIST,
         CUSTOSUP.CDFORNECLIST,
         CUSTOSUP.CDPRODLIST
      FROM
         CUSTOSUP
      WHERE
         CUSTOSUP.DTCUSTOSUP >= V_DTINIVALLIST AND
         CUSTOSUP.CDFILIAL = R1.CDLISTA)
      SELECT 'A' LETRA,
             CUSTOSUP.NRCOLEPRACOR,
             CUSTOSUP.ROWID,
             CUSTOSUP.CDFILIAL,
             CUSTOSUP.CDPRODUTO,
             CUSTOSUP.DTCUSTOSUP,
             CUSTOSUP.NRCOLEPRLIST,
             CUSTOSUP.VRPRCUSTOAF,
             CUSTOSUP.CDPRODACOR CDPRODCOT
      FROM   CUSTOSUP_X  CUSTOSUP
      WHERE
         CUSTOSUP.CDFILCOTACOR IS NOT NULL AND
         NOT EXISTS
           (SELECT /*+ INDEX(ITEMCOLE PK_ITEMCOLE)*/
               1
            FROM
               ITEMCOLE
            WHERE
               (ITEMCOLE.CDFILIAL    = CUSTOSUP.CDFILCOTACOR) AND
               (ITEMCOLE.NRCOLEPRECO = CUSTOSUP.NRCOLEPRACOR) AND
               (ITEMCOLE.CDMARCA     = CUSTOSUP.CDMARCAACOR)  AND
               (ITEMCOLE.CDREGIAO    = CUSTOSUP.CDREGIAOACOR) AND
               (ITEMCOLE.CDFORNECED  = CUSTOSUP.CDFORNECACOR) AND
               (ITEMCOLE.CDPRODUTO   = CUSTOSUP.CDPRODACOR) )
      UNION ALL
      SELECT 'B' LETRA,
             CUSTOSUP.NRCOLEPRACOR,
             CUSTOSUP.ROWID,
             CUSTOSUP.CDFILIAL,
             CUSTOSUP.CDPRODUTO,
             CUSTOSUP.DTCUSTOSUP,
             CUSTOSUP.NRCOLEPRLIST,
             CUSTOSUP.VRPRCUSTOAF,
             CUSTOSUP.CDPRODLIST CDPRODCOT
      FROM   CUSTOSUP_X  CUSTOSUP
      WHERE
         CUSTOSUP.CDFILCOTLIST IS NOT NULL AND
         NOT EXISTS
           (SELECT /*+ INDEX(ITEMCOLE PK_ITEMCOLE)*/
               1
            FROM
               ITEMCOLE
            WHERE
               (ITEMCOLE.CDFILIAL    = CUSTOSUP.CDFILCOTLIST) AND
               (ITEMCOLE.NRCOLEPRECO = CUSTOSUP.NRCOLEPRLIST) AND
               (ITEMCOLE.CDMARCA     = CUSTOSUP.CDMARCALIST) AND
               (ITEMCOLE.CDREGIAO    = CUSTOSUP.CDREGIAOLIST) AND
               (ITEMCOLE.CDFORNECED  = CUSTOSUP.CDFORNECLIST) AND
               (ITEMCOLE.CDPRODUTO   = CUSTOSUP.CDPRODLIST) ) ) LOOP

      IF C_CUSTOSUP.LETRA = 'A' THEN

        IF (C_CUSTOSUP.NRCOLEPRLIST IS NOT NULL) OR (C_CUSTOSUP.VRPRCUSTOAF IS NOT NULL) THEN
          V_CONT_COMMIT := V_CONT_COMMIT + 1;
          UPDATE CUSTOSUP
          SET
            VRCUSPRLISTP = DECODE(C_CUSTOSUP.NRCOLEPRLIST, NULL, NULL, CUSTOSUP.VRHISPRLISTP),
            VRCUSPRLISM1 = NULL,
            VRCUSPRLISM2 = NULL,
            CDFILCOTACOR = NULL,
            NRCOLEPRACOR = NULL,
            CDMARCAACOR  = NULL,
            CDREGIAOACOR = NULL,
            CDFORNECACOR = NULL,
            CDPRODACOR   = NULL
          WHERE
            ROWID = C_CUSTOSUP.ROWID;
        ELSE
          V_CONT_COMMIT := V_CONT_COMMIT + 4;
          DELETE CUSTOSUP
          WHERE
            ROWID = C_CUSTOSUP.ROWID;
        END IF;

      ELSE -- IF C_CUSTOSUP.LETRA = 'B'

        /* CURSOR: C_DELCUSTLIST
          TRATAMENTO DOS CUSTOS DE LISTA DE PREOS DA QUAL A COTAO DE LISTA
          NO MAIS EXISTE. OS CAMPOS DA COTAO DE LISTA SERO LIMPOS, MAS SE OS CAMPOS
          DE COTAO DE ACORDO E DE CUSTO DE AF NO ESTIVEREM PREENCHIDOS, O REGISTRO
          SER EXCLUDO.
        */
        IF (C_CUSTOSUP.NRCOLEPRACOR IS NOT NULL) OR (C_CUSTOSUP.VRPRCUSTOAF IS NOT NULL) THEN
          V_CONT_COMMIT := V_CONT_COMMIT + 1;
          UPDATE CUSTOSUP
          SET
            VRHISPRLISTP = NULL,
            CDFILCOTLIST = NULL,
            NRCOLEPRLIST = NULL,
            CDMARCALIST  = NULL,
            CDREGIAOLIST = NULL,
            CDFORNECLIST = NULL,
            CDPRODLIST   = NULL
          WHERE
             ROWID = C_CUSTOSUP.ROWID;
        ELSE
          V_CONT_COMMIT := V_CONT_COMMIT + 4;
          DELETE CUSTOSUP
          WHERE
            ROWID = C_CUSTOSUP.ROWID;
        END IF;

      END IF; -- C_CUSTOSUP.LETRA = 'B'

      IF (V_CONT_COMMIT >= V_QTDE_COMMIT) THEN
         V_CONT_COMMIT := 0;
         COMMIT;
      END IF;

     END LOOP; --C_CUSTOSUP
   END LOOP; --R1 FILIAIS DA LISTA !!
   DELETE FROM LISTA WHERE NRLISTA = V_LISTA_FILIAL;
   COMMIT;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('ATU_CUSTO_ITEMCOL_BKG','5.03.004'); END;
/
